#include <bcm2835.h>
#include <stdio.h>
#include "lcd.h"

#define WIDTH 192
#define HEIGHT 96

#define LEDA 23
#define RST 25
#define DC  24
char buffer[4608];

void command(char cmd) {
    bcm2835_gpio_write(DC, LOW);
    bcm2835_spi_transfer(cmd);
}

void data(char dat) {
    bcm2835_gpio_write(DC, HIGH);
    bcm2835_spi_transfer(dat);
}


void lcd_begin()
{ uint16_t Contrast=202;
  bcm2835_gpio_fsel(LEDA, BCM2835_GPIO_FSEL_OUTP);
    bcm2835_gpio_fsel(RST, BCM2835_GPIO_FSEL_OUTP);
    bcm2835_gpio_fsel(DC, BCM2835_GPIO_FSEL_OUTP);

    bcm2835_spi_begin();
    bcm2835_spi_setBitOrder(BCM2835_SPI_BIT_ORDER_MSBFIRST);     //The default
    bcm2835_spi_setDataMode(BCM2835_SPI_MODE0);                  //The default
    bcm2835_spi_setClockDivider(BCM2835_SPI_CLOCK_DIVIDER_1024);  //The default
    bcm2835_spi_chipSelect(BCM2835_SPI_CS0);                     //The default
    bcm2835_spi_setChipSelectPolarity(BCM2835_SPI_CS0, LOW);     //the default

    bcm2835_gpio_write(RST, HIGH);
    bcm2835_delay(10);
    bcm2835_gpio_write(RST, LOW);
    bcm2835_delay(10);
    bcm2835_gpio_write(RST, HIGH);

    command(0x30);	// Extension Command 1
    command(0x94);	// Sleep Out
    bcm2835_delay(50);
    command(0x31); // Extension Command 2
    command(0x32);// Analog Circuit Set
    data(0x00);
    data(0x01);
    data(0x03);
    command(0x51);  // Booster Level x10
    data(0xFA);		   //8X

    command(0x30);	// Extension Command 1
    command(0x20);   // Power Control
    data(0x0b);			// VB ON ; VR,VF ON
    command(0x81);   //Vop Control
    data(Contrast&0x3F);
    data((Contrast>>6)&0x07);	

    command(0x0C);   // Data Format Select     DO=1; LSB on top
    command(0xf0);   // Display Mode
    data(0x10);		   // Monochrome Mode 
    command(0xCA);   // Display Control
    data(0);
    data(95);			  //duty
    data(0x00);
    command(0xBC);  //ata Scan Direction
    data(0x00);		   //MY=0  

    command(0xaf);	 //Display On 

    bcm2835_gpio_write(LEDA, HIGH);

}
void lcd_clear()
{
    int i;
    for(i = 0; i < sizeof(buffer); i++)
    {
        buffer[i] = 0;
    }
}
void lcd_pixel(int x, int y, char color)
{
    if(x > WIDTH || y > HEIGHT)return ;
    if(color)
        buffer[x+(y/8)*WIDTH] |= 1<<(y%8);
    else
        buffer[x+(y/8)*WIDTH] &= ~(1<<(y%8));
}
void lcd_char1616(uint8_t x, uint8_t y, uint8_t chChar)
{
    uint8_t i, j;
    uint8_t chTemp = 0, y0 = y, chMode = 0;

    for (i = 0; i < 32; i ++) {
        chTemp = Font1612[chChar - 0x30][i];
        for (j = 0; j < 8; j ++) {
            chMode = chTemp & 0x80? 1 : 0; 
            lcd_pixel(x, y, chMode);
            chTemp <<= 1;
            y ++;
            if ((y - y0) == 16) {
                y = y0;
                x ++;
                break;
            }
        }
    }
}

void lcd_char3216(uint8_t x, uint8_t y, uint8_t chChar)
{
    uint8_t i, j;
    uint8_t chTemp = 0, y0 = y, chMode = 0;

    for (i = 0; i < 64; i ++) {
        chTemp = Font3216[chChar - 0x30][i];
        for (j = 0; j < 8; j ++) {
            chMode = chTemp & 0x80? 1 : 0; 
            lcd_pixel(x, y, chMode);
            chTemp <<= 1;
            y ++;
            if ((y - y0) == 32) {
                y = y0;
                x ++;
                break;
            }
        }
    }
}
void lcd_char(unsigned char x, unsigned char y, char acsii, char size, char mode)
{
    unsigned char i, j, y0=y;
    char temp;
    unsigned char ch = acsii - ' ';
    for(i = 0;i<size;i++) {
        if(size == 12)
        {
            if(mode)temp=Font1206[ch][i];
            else temp = ~Font1206[ch][i];
        }
        else 
        {            
            if(mode)temp=Font1608[ch][i];
            else temp = ~Font1608[ch][i];
        }
        for(j =0;j<8;j++)
        {
            if(temp & 0x80) lcd_pixel(x, y, 1);
            else lcd_pixel(x, y, 0);
            temp <<=1;
            y++;
            if((y-y0)==size)
            {
                y = y0;
                x ++;
                break;
            }
        }
    }
}
void lcd_string(uint8_t x, uint8_t y, const char *pString, uint8_t Size, uint8_t Mode)
{
    while (*pString != '\0') {       
        if (x > (WIDTH - Size / 2)) {
            x = 0;
            y += Size;
            if (y > (HEIGHT - Size)) {
                y = x = 0;
            }
        }
        
        lcd_char(x, y, *pString, Size, Mode);
        x += Size / 2;
        pString ++;
    }
}

void lcd_bitmap(unsigned char x, unsigned char y, const unsigned char *pBmp, unsigned char chWidth, unsigned char chHeight)
{
    unsigned char i, j, byteWidth = (chWidth + 7)/8;
    for(j = 0; j < chHeight; j++) {
        for(i = 0;i <chWidth;i ++) {
            if(*(pBmp + j*byteWidth + i/8) & (128 >> (i & 7))) {
                lcd_pixel(x+i, y+j, 1);
            }
        }
    }        
}

void lcd_display()
{   int i;
    uint8_t page;
    char *pBuf = (char *)buffer;
  
    command(0xf0);   // Display Mode
    data(0x10);  // Monochrome Mode	
     	  
    command(0x15);
    data(0);
    data(191);
    command(0x75);
    data(0);
    data(11);
    command(0x5c);
  
    for (page = 0; page < HEIGHT/8; page++) {  
 
        for(i = 0; i< WIDTH; i++ ) {
    	    bcm2835_gpio_write(DC, HIGH);
    	    bcm2835_spi_transfer(pBuf[i+page*WIDTH]);
        }
    }
}





void er_lcd_pixel_gray(int x, int y, char color)
{
    if(x > WIDTH || y > HEIGHT*2)return ;
    if(color)
        buffer[x+(y/8)*WIDTH] |= 1<<(y%8);
    else
        buffer[x+(y/8)*WIDTH] &= ~(1<<(y%8));
}


void er_lcd_bitmap_gray(uint8_t x,uint8_t y,const uint8_t *pBmp, uint8_t chWidth, uint8_t chHeight)
{
	uint16_t i, j,k; uint8_t page=chHeight*2/8;

    for(k = 0;k < page;k++)
    {
	for(j = 0;j < chWidth;j++)
        {
		for(i = 0;i <8;i++)
                {
			if(*(pBmp + j +k*chWidth) & (0x01 << (i & 7)))
                        {				
                            er_lcd_pixel_gray(x+j ,y + i+k*8, 1);                            
			}
                     
		}
	}		
    }
}




void er_lcd_display_gray()
{    uint8_t page,i; char *pBuf = (char *)buffer;

   command(0xf0);   // Display Mode
    data(0x11); // 4Gray  Mode
    command(0x15);
    data(0);
    data(191);
    command(0x75);
    data(0);
    data(23);
    command(0x5c);

    for (page = 0; page < HEIGHT/4; page++) {               
        for(i = 0; i< WIDTH; i++ ) {       
 	   bcm2835_gpio_write(DC, HIGH);
    	   bcm2835_spi_transfer(pBuf[i+page*WIDTH]);
        }        
    }

}

   



















